from __future__ import with_statement # requires Python 2.5
import os
import struct
import sys

def unpack_trs(trs_file, text_file):
    text_data = ''
    with file(trs_file, 'rb') as trs:
        assert trs.read(4) == "ETRS"
        (size,) = struct.unpack('>I', trs.read(4))
        assert trs.read(8) == "\x00" * 8
        for c in trs.read():
            text_data += chr(ord(c) ^ 0xCC)
        assert len(text_data) == size - 8

    with file(text_file, 'wb') as text:
        text.write(text_data)

def pack_trs(text_file, trs_file):
    text_data = ''
    with file(text_file, 'rb') as text:
        for c in text.read():
            text_data += chr(ord(c) ^ 0xCC)

    with file(trs_file, 'wb') as trs:
        trs.write("ETRS")
        trs.write(struct.pack('>I', len(text_data) + 8))
        trs.write("\x00" * 8) # don't know what this is
        trs.write(text_data)

def help():
    help = """
    trspack helper util.

    usage:
    trspack [-p|-u] file1 file2

    Aids in translating movie subtitles for LucasArts games, such as The Dig.

    To unpack a TRS file, pass argument "-u", followed by the input
    TRS file name, and the output text file name.

    To pack a TRS file, pass argument "-p", followed by the the input
    text file name, and the output TRS file name.
"""
    return help

def main_trspack():
    args = sys.argv[1:]
    if (len(args) < 3 or
        not os.path.isfile(args[1])):
        print help()
        return 1
    if args[0] == "-u":
        unpack_trs(args[1], args[2])
    elif args[0] == "-p":
        pack_trs(args[1], args[2])
    else:
        print help()
        return 1
    return 0


if __name__ == "__main__": sys.exit(main_trspack())