from __future__ import with_statement # requires Python 2.5
import wx
from os import getcwdu
from os.path import basename, join, isfile
import subprocess # for external tools
import traceback
import StringIO

import ssconst
import ssglob
import ssimport
import ssexport
import ssspeechmap
from ssmisc import SSException
from ui.ids import *
from ui.dialogs import *
from ui.frames import MainFrame

def catchUnhandledExceptions(func):
    def inner_func(self, *args, **kwds):
        try:
            func(self, *args, **kwds)
        except Exception, e:
            fp = StringIO.StringIO()
            traceback.print_exc(file=fp)

            traceback.print_exc()
            alert = wx.MessageDialog(self.frame,
                            "An unhandled exception was thrown, please contact support.\n" +
                            "\nError:\n" + fp.getvalue(),
                            "Unhandled Exception",
                            wx.OK|wx.ICON_ERROR)
            alert.ShowModal()
            alert.Destroy()
    return inner_func

#-------------
# Start convoluted ScummSpeaksApp class.
#-------------

class ScummSpeaksApp(wx.App):
    def OnInit(self):
        self.frame = MainFrame(None, -1, "Hello from OH GOD MY BRAINS", size=wx.Size(640, 480), moofpar=self)
        self.frame.Show(True)
        self.SetTopWindow(self.frame)
        self.projfile = "DEPRECATED"
        self.speech_map_filename = ""
        self.text_filename = ""
        self.dirtyCount = 0
        self.ftd = None

        # Set up events
        wx.EVT_MENU(self.frame, ID_NEWSPEECHMAP, self.menuNewSpeechMap)
        wx.EVT_MENU(self.frame, ID_OPENSPEECHMAP, self.menuOpenSpeechMap)
        wx.EVT_MENU(self.frame, ID_OPENTEXT, self.menuOpenText)
        wx.EVT_MENU(self.frame, ID_SAVESPEECHMAP, self.menuSaveSpeechMap)
        wx.EVT_MENU(self.frame, ID_SAVESPEECHMAPAS, self.menuSaveSpeechMapAs)
        wx.EVT_MENU(self.frame, ID_SAVETEXT, self.menuSaveText)
        wx.EVT_MENU(self.frame, ID_SAVETEXTAS, self.menuSaveTextAs)
        wx.EVT_MENU(self.frame, ID_IMPORTSOUNDS, self.menuImportSounds)
        wx.EVT_MENU(self.frame, ID_EXPORTTOGAMERESOURCES, self.menuExportToGameResources)
        wx.EVT_MENU(self.frame, ID_EXPORTSOUNDS, self.menuExportSounds)
        wx.EVT_MENU(self.frame, ID_EXPORTTEXT, self.menuExportText)
        wx.EVT_MENU(self.frame, ID_EXPORTTEXTSCRIPT, self.menuExportTextScript)
        wx.EVT_MENU(self.frame, wx.ID_EXIT, self.menuExit)


        wx.EVT_MENU(self.frame, ID_EDITFIND, self.menuFind)
        wx.EVT_MENU(self.frame, ID_EDITGOTO, self.menuGoto)

        wx.EVT_MENU(self.frame, ID_EXTRACTTEXT, self.menuExtractText)
        wx.EVT_MENU(self.frame, ID_INSERTTEXT, self.menuInsertText)

        wx.EVT_MENU(self.frame, wx.ID_HELP, self.menuHelp)
        wx.EVT_MENU(self.frame, ID_ABOUT, self.menuAbout)

        wx.EVT_CLOSE(self.frame, self.closeFrame)
        wx.EVT_BUTTON(self.frame, ID_BUTTON_INSERT, self.buttonMapSoundToText)
        wx.EVT_BUTTON(self.frame, ID_BUTTON_REMOVE, self.buttonUnmapSoundFromText)
        wx.EVT_BUTTON(self.frame, ID_BUTTON_EDITTEXT, self.buttonEditText)
        wx.EVT_BUTTON(self.frame, ID_BUTTON_TEXTDESC, self.buttonTextDescription)
        wx.EVT_BUTTON(self.frame, ID_BUTTON_ADDVOC, self.buttonAddSound)
        wx.EVT_BUTTON(self.frame, ID_BUTTON_REMVOC, self.buttonRemoveSound)
        wx.EVT_BUTTON(self.frame, ID_BUTTON_REPVOC, self.buttonReplaceSound)
        wx.EVT_BUTTON(self.frame, ID_BUTTON_VOCDESC, self.buttonSoundDescription)
        wx.EVT_LIST_KEY_DOWN(self.frame, -1, self.keyListener)

        return True

    # Most functionality is buried in the GUI event handling, unfortunately,
    #  so there may be some calls to functions that require an event but are
    #  are passed either "None" or the original menu event.

    # ---------
    # Start menus
    # ---------
    @catchUnhandledExceptions
    def menuNewSpeechMap(self, event):
        text_path = None
        resource_path = None
        od = NewSpeechMapDialog(self.frame)
        while True:
            result = od.ShowModal()
            if result != wx.ID_OK:
                od.Destroy()
                return
            text_path = od.getTextPath()
            resource_path = od.getResourcePath()
            if isfile(text_path) and isfile(resource_path):
                break
            else:
                md = wx.MessageDialog(self.frame,
                    "Specified files not found, check paths again.",
                    "File not found",
                    wx.OK|wx.ICON_ERROR)
                md.ShowModal()
                md.Destroy()
        od.Destroy()

        # Get our sound resource importer
        sound_importer = ssimport.SoundResourceImporterFactory(resource_path)
        if sound_importer is None:
            md = wx.MessageDialog(self.frame,
                    "The sound resource specified is either not recognised "
                    "or not supported.",
                    "Unrecognised sound resource",
                    wx.OK|wx.ICON_ERROR)
            md.ShowModal()
            md.Destroy()
            return

        # Import the text data
        # (possibly a bit ugly because it relies on the resource type from the
        #  sound importer)
        text_importer = ssimport.TextImporterFactory(sound_importer.resource_type)
        text_data, text_sound_map = text_importer.import_from_resource(text_path)
        ssglob.gTextData = text_data
        self.frame.list_text.CreateList()
        self.frame.list_text.RefreshViewableItems()

        # Import the sound resource data
        sounds, resource_type, compression_type = sound_importer.import_from_resource(resource_path)
        speech_map = ssspeechmap.SpeechMap()
        speech_map.text_path = text_path
        speech_map.resource_type = resource_type
        speech_map.compression_type = compression_type
        speech_map.sounds = sounds
        speech_map.import_text_sound_map(text_sound_map)
  
        ssglob.gSpeechMap = speech_map
        self.frame.list_sounds.CreateList()
        self.frame.list_sounds.RefreshViewableItems()
        self.markAsDirty(ssglob.iSpeechMapDirty)
        self.frame.statusbar.SetStatusText("Created new speech map from %s and %s" %
                    (basename(text_path), basename(resource_path)))

    @catchUnhandledExceptions
    def menuOpenSpeechMap(self, event):
        if True in ssglob.lDirty:
            wd = wx.MessageDialog(self.frame,
                                  "Save modified files before loading a new speech map?",
                                  "ScummSpeaks",
                                  wx.YES_NO|wx.CANCEL|wx.ICON_EXCLAMATION)
            test = wd.ShowModal()
            if test == wx.ID_CANCEL:
                return
            elif test == wx.ID_YES:
                if ssglob.lDirty[ssglob.iTextDirty]:
                    self.menuSaveText(None)
                if ssglob.lDirty[ssglob.iSpeechMapDirty]:
                    self.menuSaveSpeechMap(None)
                # User could click "Cancel" during any "Save As..." dialog,
                #  so again check if things are (still) dirty.
                if True in ssglob.lDirty:
                    return

        od = wx.FileDialog(self.frame,
                         "Open Speech Map",
                         wildcard="ScummSpeaks speech map (*.xml)|*.xml|All Files|*.*",
                         style=wx.OPEN|wx.FILE_MUST_EXIST|wx.CHANGE_DIR)
        result = od.ShowModal()
        speech_map_filename = od.GetPath()
        od.Destroy()
        if result != wx.ID_OK:
            return
        
        speech_map = ssspeechmap.SpeechMap()
        speech_map.load_from_file(speech_map_filename)

        # Import the text data
        try:
            # we ignore the text_sound_map, and rely on whatever's in the
            #  existing speech map.
            text_importer = ssimport.TextImporterFactory(speech_map.resource_type)
            text_data, text_sound_map = text_importer.import_from_resource(speech_map.text_path)
        except Exception, e:
            ssglob.gTextData = None
            md = wx.MessageDialog(self.frame,
                "The dialogue text data could not be read. Check that the file exists.\n" +
                "Error:\n" +
                str(e),
                "Failed to read dialogue text data",
                wx.OK|wx.ICON_ERROR)
            md.ShowModal()
            md.Destroy()
            return

        ssglob.gTextData = text_data
        ssglob.gSpeechMap = speech_map
        self.frame.list_text.CreateList()
        self.frame.list_text.RefreshViewableItems()
        self.frame.list_sounds.CreateList()
        self.frame.list_sounds.RefreshViewableItems()

        self.text_filename = speech_map.text_path
        self.speech_map_filename = speech_map_filename
        self.frame.statusbar.SetStatusText("Loaded speech map from " + basename(speech_map_filename))

    @catchUnhandledExceptions
    def menuSaveSpeechMap(self, event, filename=None):
        if filename is None: # no override
            if self.speech_map_filename == "": # but no default!
                self.menuSaveSpeechMapAs(event)
                return # this function must be run again through the above call to Save As
            filename = self.speech_map_filename

        ssglob.gSpeechMap.save_to_file(filename)

        self.markAsClean(ssglob.iSpeechMapDirty)
        self.speech_map_filename = filename
        self.frame.statusbar.SetStatusText("Project saved as " + filename)

    @catchUnhandledExceptions
    def menuSaveSpeechMapAs(self, event):
        if ssglob.gSpeechMap is None:
            self.frame.statusbar.SetStatusText("You must have a speech map open to save one!")
            return
        sd = wx.FileDialog(self.frame,
                         "Save Speech Map",
                         wildcard="ScummSpeaks speech map (*.xml)|*.xml|All Files|*.*",
                         style=wx.SAVE|wx.OVERWRITE_PROMPT|wx.CHANGE_DIR)
        if sd.ShowModal() == wx.ID_OK:
            self.menuSaveSpeechMap(event, sd.GetPath())
        sd.Destroy()

    # --- Start Text stuff ----
    @catchUnhandledExceptions
    def menuOpenText(self, event):
        if ssglob.gSpeechMap is None:
            self.frame.statusbar.SetStatusText("You must have a speech map in order to change the associated dialogue text file.")
            return
        if ssglob.lDirty[ssglob.iSpeechMapDirty]:
            wd = wx.MessageDialog(self.frame,
                                  "Save modified/changed dialogue text file?",
                                  "ScummSpeaks",
                                  wx.YES_NO|wx.CANCEL|wx.ICON_EXCLAMATION)
            test = wd.ShowModal()
            if test == wx.ID_CANCEL:
                return
            elif test == wx.ID_YES:
                self.menuSaveText(None)
                # User could click "Cancel" during any "Save As..." dialog,
                #  so again check if things are (still) dirty.
                if ssglob.lDirty[ssglob.iSpeechMapDirty]:
                    return

        od = wx.FileDialog(self.frame,
                         "Open Text",
                         wildcard="Dialogue text (*.txt)|*.txt|All Files|*.*",
                         style=wx.OPEN|wx.FILE_MUST_EXIST|wx.CHANGE_DIR)
        result = od.ShowModal()
        text_fname = od.GetPath()
        od.Destroy()
        if result != wx.ID_OK:
            return

        # Import the text data
        try:
            # we ignore the text_sound_map, and rely on whatever's in the
            #  existing speech map.
            text_importer = ssimport.TextImporterFactory(ssglob.gSpeechMap.resource_type)
            text_data, text_sound_map = text_importer.import_from_resource(text_fname)
        except Exception, e:
            ssglob.gTextData = None
            md = wx.MessageDialog(self.frame,
                "The dialogue text data could not be read. Check that the file exists.\n" +
                "Error:\n" +
                str(e),
                "Failed to read dialogue text data",
                wx.OK|wx.ICON_ERROR)
            md.ShowModal()
            md.Destroy()
            return
        ssglob.gTextData = text_data
        ssglob.gSpeechMap.text_path = text_fname
        self.text_filename = text_fname

        self.frame.list_text.CreateList()
        self.frame.list_text.RefreshViewableItems()
        self.markAsDirty(ssglob.iSpeechMapDirty) # since it stores the text path
        self.frame.statusbar.SetStatusText("Text successfully changed to " + text_fname)

    @catchUnhandledExceptions
    def menuSaveText(self, event, filename=None):
        if filename is None:
            if self.text_filename == "":
                self.menuSaveTextAs(event)
                return
            filename = self.text_filename

        ssglob.gTextData.save_to_file(filename)
        self.markAsClean(ssglob.iTextDirty)
        self.frame.statusbar.SetStatusText("Text successfully saved to " + filename)

    @catchUnhandledExceptions
    def menuSaveTextAs(self, event):
        if self.text_filename == "":
            self.frame.statusbar.SetStatusText("Must open an existing text file before trying to save it.")
            return
        sd = wx.FileDialog(self.frame,
                           "Save output text",
                           wildcard="TXT files (*.txt)|*.txt|All Files|*.*",
                           style=wx.SAVE|wx.OVERWRITE_PROMPT|wx.CHANGE_DIR)
        if sd.ShowModal() == wx.ID_OK:
            self.text_filename = sd.GetPath()
            self.menuSaveText(event, sd.GetPath())
        sd.Destroy()
    # --- End Text stuff ----

    # --- Start Import stuff ----
    @catchUnhandledExceptions
    def menuImportSounds(self, event):
        if ssglob.gSpeechMap is None or ssglob.gTextData is None:
            self.frame.statusbar.SetStatusText("Must have a speech map and text file open to import sounds.")
            return

        # Warn user that all sounds will be replaced
        alert = wx.MessageDialog(self.frame,
                    "All existing sounds and their mappings will be replaced with the imported sounds.",
                    "Warning",
                    wx.OK|wx.CANCEL|wx.ICON_EXCLAMATION)
        result = alert.ShowModal()
        alert.Destroy()
        if result != wx.ID_OK:
            return

        # Choose a folder
        fd = wx.DirDialog(self.frame, "Import sounds from...", getcwdu())
        if fd.ShowModal() != wx.ID_OK:
            return
        in_path = fd.GetPath()

        # Export all sounds.
        try:
            self.frame.statusbar.SetStatusText("Importing sound files...")
            ssglob.gSpeechMap.import_sounds(in_path)

            self.markAsDirty(ssglob.iSpeechMapDirty)
            self.frame.list_text.CreateList()
            self.frame.list_sounds.CreateList()
            alert = wx.MessageDialog(self.frame,
                      "Sound files were successfully imported from %s" %
                      in_path,
                      "Import successful",
                      wx.OK|wx.ICON_INFORMATION)
            alert.ShowModal()
            alert.Destroy()
            self.frame.statusbar.SetStatusText("Sound files imported successfully from " + in_path)
        except Exception, e:
            traceback.print_exc()
            ed = wx.MessageDialog(self.frame,
                                  str(e),
                                  "Error importing sounds",
                                  wx.OK|wx.ICON_ERROR)
            ed.ShowModal()
            ed.Destroy()
            self.frame.statusbar.SetStatusText("")
    # --- End Import stuff ----

    # --- Start Export stuff ----
    @catchUnhandledExceptions
    def menuExportToGameResources(self, event):
        if ssglob.gSpeechMap is None or ssglob.gTextData is None:
            self.frame.statusbar.SetStatusText("Must have a speech map and text file open to export to game resources.")
            return

        text_path = None
        resource_path = None
        od = ExportToGameResourcesDialog(self.frame)
        if od.ShowModal() != wx.ID_OK:
            od.Destroy()
            return

        text_path = od.getTextPath()
        resource_path = od.getResourcePath()
        od.Destroy()

        if isfile(text_path) or isfile(resource_path):
            alert = wx.MessageDialog(self.frame,
                "Files with the specified names already exist, do you want to overwrite them?",
                "Files already exist",
                wx.OK|wx.CANCEL|wx.ICON_EXCLAMATION)
            if alert.ShowModal() != wx.ID_OK:
                alert.Destroy()
                return
            alert.Destroy()

        try:
            # Export sounds
            sound_exporter = ssexport.ResourceExporterFactory(ssglob.gSpeechMap)
            id_to_game_offset_map = sound_exporter.export_to_resource(resource_path, ssglob.gSpeechMap)

            # Export text data
            text_exporter = ssexport.TextExporterFactory(ssglob.gSpeechMap)
            text_exporter.export_to_resource(ssglob.gTextData, text_path, ssglob.gSpeechMap, id_to_game_offset_map)

            alert = wx.MessageDialog(self.frame,
                "Text and sounds were successfully exported to game resources.\n" +
                "(Remember to use ScummTr to insert the new text!)",
                "Export Successful",
                wx.OK|wx.ICON_INFORMATION)
            alert.ShowModal()
            alert.Destroy()
            self.frame.statusbar.SetStatusText("Successfully exported to game resources.")

        except Exception, e:
            traceback.print_exc()
            ed = wx.MessageDialog(self.frame,
                                  str(e),
                                  "Error exporting to game resources",
                                  wx.OK|wx.ICON_ERROR)
            ed.ShowModal()
            ed.Destroy()
            self.frame.statusbar.SetStatusText("")

    @catchUnhandledExceptions
    def menuExportSounds(self, event):
        if ssglob.gSpeechMap is None or ssglob.gTextData is None:
            self.frame.statusbar.SetStatusText("Must have a speech map and text file open to export sounds.")
            return

        # Choose a folder
        fd = wx.DirDialog(self.frame, "Export sounds to...", getcwdu())
        if fd.ShowModal() != wx.ID_OK:
            return
        out_path = fd.GetPath()
        if len(out_path) > 219: # filename will be at most 36 chars
            ed = wx.MessageDialog(self.frame,
                                  "Output path is too long; please choose a shorter path, under 220 characters.",
                                  "Error exporting sounds",
                                  wx.OK|wx.ICON_ERROR)
            ed.ShowModal()
            ed.Destroy()
            return

        # Export all sounds.
        try:
            self.frame.statusbar.SetStatusText("Exporting sound files...")
            ssglob.gSpeechMap.export_sounds(out_path, ssglob.gTextData)
            alert = wx.MessageDialog(self.frame,
                      "Sound files were successfully exported to %s" %
                      out_path,
                      "Export successful",
                      wx.OK|wx.ICON_INFORMATION)
            alert.ShowModal()
            alert.Destroy()
            self.frame.statusbar.SetStatusText("Sound files exported successfully to " + out_path)
        except Exception, e:
            traceback.print_exc()
            ed = wx.MessageDialog(self.frame,
                                  str(e),
                                  "Error exporting sounds",
                                  wx.OK|wx.ICON_ERROR)
            ed.ShowModal()
            ed.Destroy()
            self.frame.statusbar.SetStatusText("")

    @catchUnhandledExceptions
    def menuExportText(self, event):
        if ssglob.gSpeechMap is None or ssglob.gTextData is None:
            self.frame.statusbar.SetStatusText("A text file must be open to export selected text.")
            return
        etd = ExportTextDialog(self.frame)
        test = etd.ShowModal()
        if test == wx.ID_CANCEL:
            return
        match_text = etd.getMatchText()
        with file(etd.getTextFilename(), 'w+') as out_file:
            for id, comment in ssglob.gSpeechMap.text_comments.items():
                if match_text in comment:
                    out_file.write(ssglob.gTextData[id])
        self.frame.statusbar.SetStatusText("Text with \"" + match_text + "\" in description saved to "+ out_file.name +".")

    @catchUnhandledExceptions
    def menuExportTextScript(self, event):
        if ssglob.gSpeechMap is None or ssglob.gTextData is None:
            self.frame.statusbar.SetStatusText("A text file and speech map must be open to export a text script.")
            return

        etd = ExportTextScriptDialog(self.frame)
        test = etd.ShowModal()
        if test == wx.ID_CANCEL:
            return
        start, end = etd.getRange()
        ignoretext = etd.ignoreText()
        containstext = etd.containsText()
        outFile = file(etd.getTextFilename(), 'w+')

        for i in xrange(start, end + 1): # inclusive range
            # Get the description for the line of text.
            try:
                desc = ssglob.gSpeechMap.text_comments[i]
            except KeyError:
                continue # no description, go to next line of text
            # Implicitly ignore lines with no description, and ignore lines whose
            #  descriptions contain the text we want to ignore.
            # Also ignore lines that do not contain the "must contain" text.
            if (not ignoretext is None and ignoretext.lower() in desc.lower()) or \
               (not containstext is None and not containstext.lower() in desc.lower()):
                continue
            # Get the line of text
            text = ssglob.gTextData[i]
            # Interpret some script data
            text = text.replace("\\xFF\\x03", "\n").replace("^", "...")
            # Strip the remaining script data (all hex values)
            while text.find("\\x") != -1:
                pos = text.find("\\x")
                text = text[:pos] + text[pos + 4:]
            outFile.write("[" + desc + " - #" + str(i).zfill(4) + "]\n" + text + "\n")

        outFile.close()
        self.frame.statusbar.SetStatusText("Text between lines " + str (start) + "-" + str(end) + " saved to "+ outFile.name +".")
    # --- End Export stuff ----
    @catchUnhandledExceptions
    def menuExit(self, event):
        self.frame.Close(False)

    #----------
    # Edit menu
    @catchUnhandledExceptions
    def menuGoto(self, event):
        if ssglob.gTextData is None:
            self.frame.statusbar.SetStatusText("Must open an existing text file before exploring it.")
            return
        gd = wx.TextEntryDialog(self.frame,
                                "Enter the line number",
                                caption="Go to line")
        test = gd.ShowModal()
        if test == wx.ID_OK:
            try:
                line = int(gd.GetValue())
                if type(line) == long:
                    self.frame.statusbar.SetStatusText("Line number entered is too large.")
                elif self.frame.list_text.EnsureVisible(line):
                    self.frame.list_text.SelectItemManual(line)
                else:
                    self.frame.statusbar.SetStatusText("Invalid line number entered.")
            except ValueError: # takes care of non-numbers for input
                self.frame.statusbar.SetStatusText("Invalid line number entered.")
        gd.Destroy()

    @catchUnhandledExceptions
    def menuFind(self, event):
        if ssglob.gTextData is None:
            self.frame.statusbar.SetStatusText("Must open an existing text file before exploring it.")
            return
        self.ftd = FindTextDialog(self.frame)
        self.ftd.Show()

    #-----------
    # Tools menu
    @catchUnhandledExceptions
    def menuExtractText(self, event):
        scummtr_dialog = ScummTrDialog(self.frame)
        scummtr_dialog.selectExtract()
        if scummtr_dialog.ShowModal() == wx.ID_OK:
            switches = scummtr_dialog.constructCommandSwitches()
            pfile = subprocess.Popen('"' + join(ssconst.strScummtrPath, ssconst.strScummtrName) + '" ' + \
                                     switches,
                                     stdin=subprocess.PIPE,
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
            retcode = pfile.wait()
            error_msg = pfile.stderr.read()
            if error_msg == '':
                alert = wx.MessageDialog(self.frame,
                                      "The dialogue text was successfully extracted!",
                                      "Text Extracted OK",
                                      wx.OK|wx.ICON_INFORMATION)
            else:
                alert = wx.MessageDialog(self.frame,
                                      "ScummTr failed with the following error message. " +
                                      "Check your switches and paths.\n\n" +
                                      "%s" % error_msg,
                                      "Text Extraction Failed",
                                      wx.OK|wx.ICON_ERROR)
            alert.ShowModal()
            alert.Destroy()
        scummtr_dialog.Destroy()

    @catchUnhandledExceptions
    def menuInsertText(self, event):
        if not self.checkDirtyText():
            return

        scummtr_dialog = ScummTrDialog(self.frame)
        scummtr_dialog.selectInsert()
        if scummtr_dialog.ShowModal() == wx.ID_OK:
            switches = scummtr_dialog.constructCommandSwitches()
            pfile = subprocess.Popen('"' + join(ssconst.strScummtrPath, ssconst.strScummtrName) + '" ' + \
                                     switches,
                                     stdin=subprocess.PIPE,
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
            retcode = pfile.wait()
            error_msg = pfile.stderr.read()
            if error_msg == '':
                alert = wx.MessageDialog(self.frame,
                                      "The dialogue text was successfully inserted!",
                                      "Text Inserted OK",
                                      wx.OK|wx.ICON_INFORMATION)
            else:
                alert = wx.MessageDialog(self.frame,
                                      "ScummTr failed with the following error message. " +
                                      "Check your switches and paths.\n\n" +
                                      "%s" % error_msg,
                                      "Text Insertion Failed",
                                      wx.OK|wx.ICON_ERROR)
            alert.ShowModal()
            alert.Destroy()
        scummtr_dialog.Destroy()
        
    #----------
    # Help menu
    @catchUnhandledExceptions
    def menuHelp(self, event):
        # I don't think this is the right way to do it
        #subprocess.Popen('README.TXT', shell=True)
        # Above was crap. I'm going to just redirect the user.
        ad = wx.MessageDialog(self.frame,
                              "Please see documents in the program's " +
                              "'docs' directory, or contact Laurence directly: \n\n" +
                              "jestarjokin@jestarjokin.net",
                              'Help',
                              style=wx.OK|wx.ICON_INFORMATION)
        ad.ShowModal()

    @catchUnhandledExceptions
    def menuAbout(self, event):
        ad = wx.MessageDialog(self.frame,
                              'ScummSpeaks ' + ssconst.strVersion + '\n\n' + \
                              'Laurence Dougal Myers\n' + \
                              'www.jestarjokin.net\n\n' + \
                              'Thanks to:\n' + \
                              'Jimmi Thogersen (Serge) for ScummRev\n' + \
                              'Thomas Combeleran for ScummRp and ScummTr\n' + \
                              'The ScummVM team',
                              'About',
                              style=wx.OK|wx.ICON_INFORMATION)
        ad.ShowModal()
        ad.Destroy()

    # ---------
    # Start buttons
    # ---------
    @catchUnhandledExceptions
    def buttonMapSoundToText(self, event):
        if self.frame.list_text.selected is None or self.frame.list_sounds.selected is None:
            self.frame.statusbar.SetStatusText("Must select both a line of text, and a sound to use.")
            return

        try:
            ssglob.gSpeechMap.map_sound_to_text(
                self.frame.list_sounds.SelectedToIndex(),
                self.frame.list_text.SelectedToIndex())

            self.frame.list_text.RefreshSelectedItem()
            self.frame.list_text.SelectNextItem()
            self.frame.list_sounds.RefreshSelectedItem()
            self.frame.list_sounds.SelectNextItem()
            self.markAsDirty(ssglob.iSpeechMapDirty)
            self.frame.statusbar.SetStatusText("Speech text successfully added/updated.")
        except SSException:
            self.frame.statusbar.SetStatusText(
                ("Could not map sound #%s to line #%s " +
                "because the line is already mapped.") %
                (self.frame.list_sounds.SelectedToIndex(), self.frame.list_text.SelectedToIndex()))

    @catchUnhandledExceptions
    def buttonUnmapSoundFromText(self, event):
        line_id = self.frame.list_text.SelectedToIndex()
        if line_id is None:
            self.frame.statusbar.SetStatusText("Must select a line of text to unmap a sound.")
            return

        try:
            ssglob.gSpeechMap.unmap_sound_from_text(line_id)
            self.frame.list_text.RefreshSelectedItem()
            self.frame.list_sounds.RefreshViewableItems()
            self.markAsDirty(ssglob.iSpeechMapDirty)
            self.frame.statusbar.SetStatusText("Speech text successfully removed from line #%s." % (line_id,))
        except SSException:
            self.frame.statusbar.SetStatusText("No sound is mapped to line #%s." % (line_id,))

    @catchUnhandledExceptions
    def buttonAddSound(self, event):
        if ssglob.gSpeechMap is None:
            self.frame.statusbar.SetStatusText("No Speech Map is currently open.")
            return
        od = wx.FileDialog(self.frame,
                           "Open Sound",
                           wildcard="All Sound Files|*.flac;*.mp3;*.ogg|" + \
                                    "FLAC files (*.flac)|*.flac|" + \
                                    "MP3 files (*.mp3)|*.mp3|" + \
                                    "OGG files (*.ogg)|*.ogg|" + \
                                    "All Files|*.*",
                           style=wx.OPEN|wx.FILE_MUST_EXIST|wx.MULTIPLE)
        if od.ShowModal() == wx.ID_OK:
            filenames = od.GetPaths()
            counter = 0

            for sound_name in filenames:
                try:
                    ssglob.gSpeechMap.add_sound(sound_name)
                    counter += 1
                except SSException, sse:
                    ed = wx.MessageDialog(self.frame,
                                          str(sse),
                                          "Error adding sound",
                                          wx.OK|wx.ICON_ERROR)
                    ed.ShowModal()
            self.frame.list_sounds.CreateList() # reload the list
            self.frame.list_sounds.EnsureVisible(len(ssglob.gSpeechMap) - 1)
            if counter != 0:
                self.markAsDirty(ssglob.iSpeechMapDirty)
                self.frame.statusbar.SetStatusText("Successfully added " + str(counter) + "/" + str(len(filenames)) + " sound files.")
            else:
                self.frame.statusbar.SetStatusText("Could not add any of the selected files.")

    @catchUnhandledExceptions
    def buttonRemoveSound(self, event):
        if ssglob.gSpeechMap is None:
            self.frame.statusbar.SetStatusText("No Speech Map open.")
            return
        elif self.frame.list_sounds.selected is None:
            self.frame.statusbar.SetStatusText("Must select a sound to remove.")
            return
        wd = wx.MessageDialog(self.frame,
                              "Are you sure you want to remove the currently selected sound file?\n",
                              "Remove sound",
                              wx.YES_NO|wx.ICON_EXCLAMATION)
        test = wd.ShowModal()
        if test == wx.ID_YES:
            top_visible_item = self.frame.list_sounds.GetTopItem()
            sound_num = self.frame.list_sounds.SelectedToIndex()
            sound_id = ssglob.gSpeechMap.sounds[sound_num].id
            ssglob.gSpeechMap.remove_sound(sound_num)

            self.frame.list_sounds.CreateList()
            self.frame.list_sounds.EnsureVisibleAtTop(top_visible_item)
            # also need to refresh the text view
            self.frame.list_text.RefreshViewableItems()
            self.markAsDirty(ssglob.iSpeechMapDirty)
            self.frame.statusbar.SetStatusText("Successfully removed sound #%s with ID %s." % (sound_num, sound_id))

    @catchUnhandledExceptions
    def buttonReplaceSound(self, event):
        if ssglob.gSpeechMap is None:
            self.frame.statusbar.SetStatusText("No Speech Map open.")
            return
        od = wx.FileDialog(self.frame,
                           "Open Sound",
                           wildcard="All Sound Files|*.flac;*.mp3;*.ogg|" + \
                                    "FLAC files (*.flac)|*.flac|" + \
                                    "MP3 files (*.mp3)|*.mp3|" + \
                                    "OGG files (*.ogg)|*.ogg|" + \
                                    "All Files|*.*",
                           style=wx.OPEN|wx.FILE_MUST_EXIST|wx.MULTIPLE)
        if od.ShowModal() == wx.ID_OK:

            filenames = od.GetPaths()
            counter = 0

            if len(filenames) > 1:
                self.frame.statusbar.SetStatusText("Can only replace one sound at a time.")
                return

            sound_num = self.frame.list_sounds.SelectedToIndex()

            for sound_name in filenames:
                try:
                    ssglob.gSpeechMap.replace_sound(sound_num, sound_name)
                    counter += 1
                except SSException, sse:
                    ed = wx.MessageDialog(self.frame,
                                          str(sse),
                                          "Error replacing sound",
                                          wx.OK|wx.ICON_ERROR)
                    ed.ShowModal()
            #self.frame.list_sounds.CreateList()
            #self.frame.list_sounds.EnsureVisible(sound_num)
            self.frame.list_sounds.RefreshSelectedItem()
            # also need to refresh the text view
            self.frame.list_text.RefreshViewableItems()
            if counter != 0:
                self.markAsDirty(ssglob.iSpeechMapDirty)
                self.frame.statusbar.SetStatusText("Successfully replaced sound %s with %s." %
                                                    (sound_num, sound_name))
            else:
                self.frame.statusbar.SetStatusText("Could not replace the selected sound.")

    @catchUnhandledExceptions
    def buttonTextDescription(self, event):
        if self.frame.list_text.selected is None:
            self.frame.statusbar.SetStatusText("Must select a line of text to describe.")
            return

        line_num = self.frame.list_text.SelectedToIndex()
        ed = wx.TextEntryDialog(self.frame,
                                "Enter a description or comment for the selected text.",
                                caption="Enter description",
                                defaultValue=ssglob.gSpeechMap.text_comments.get(line_num, ''))
        if ed.ShowModal() == wx.ID_OK:
            comment_id = line_num
            if ed.GetValue() != '':
                ssglob.gSpeechMap.text_comments[comment_id] = ed.GetValue()
            elif comment_id in ssglob.gSpeechMap.text_comments:
                del ssglob.gSpeechMap.text_comments[comment_id]
            self.frame.list_text.RefreshItem(line_num)
            self.frame.list_text.SelectNextItem()
            self.markAsDirty(ssglob.iSpeechMapDirty)
        ed.Destroy()

    @catchUnhandledExceptions
    def buttonSoundDescription(self, event):
        if self.frame.list_sounds.selected is None:
            self.frame.statusbar.SetStatusText("Must select a sound to describe.")
            return

        item = self.frame.list_sounds.SelectedToIndex()
        ed = wx.TextEntryDialog(self.frame,
                                "Enter a description or comment for the selected sound.",
                                caption="Enter description",
                                defaultValue=ssglob.gSpeechMap.sound_comments.get(
                                            ssglob.gSpeechMap.sounds[item].id, ''))
        if ed.ShowModal() == wx.ID_OK:
            comment_id = ssglob.gSpeechMap.sounds[item].id
            if ed.GetValue() != '':
                ssglob.gSpeechMap.sound_comments[comment_id] = ed.GetValue()
            elif comment_id in ssglob.gSpeechMap.sound_comments:
                del ssglob.gSpeechMap.sound_comments[comment_id]
            self.frame.list_sounds.RefreshItem(item)

    @catchUnhandledExceptions
    def buttonEditText(self, event):
        if self.frame.list_text.selected == None:
            self.frame.statusbar.SetStatusText("Must select a line of text to edit.")
            return

        line_num = self.frame.list_text.SelectedToIndex()
        ed = wx.TextEntryDialog(self.frame,
                                "Edit the text to suit your whim, you desecrating heathen.",
                                caption="Edit text",
                                defaultValue=ssglob.gTextData[line_num].rstrip('\n'))
        if ed.ShowModal() == wx.ID_OK:
            ssglob.gTextData[line_num] = ed.GetValue()
            self.frame.list_text.RefreshItem(line_num)
            self.frame.list_text.SelectNextItem()
            self.markAsDirty(ssglob.iTextDirty)
        ed.Destroy()

    @catchUnhandledExceptions
    def buttonFindText(self, ttf):
        textToMatch = unicode(self.ftd.getSearchText()) # so we don't get errors comparing
        start = self.frame.list_text.selected
        if start == None:
            start = 0
        elif start + 1 == (len(ssglob.gTextData) - 1):
            self.frame.statusbar.SetStatusText("Text \"" + textToMatch + "\" not found.")
            return
        try:
            for i in xrange(start + 1, len(ssglob.gTextData)):
                if self.ftd.shouldSearchDesc():
                    try:
                        line = ssglob.gSpeechMap.text_comments[
                            self.frame.list_text.indexMap[i]
                        ]
                    except KeyError:
                        continue
                else:
                    # This is supposed to prevent weird unicode errors...
                    line = unicode(ssglob.gTextData[
                        self.frame.list_text.indexMap[i]
                    ], errors='ignore')
                if textToMatch.lower() in line.lower():
                    if self.frame.list_text.EnsureVisible(i):
                        self.frame.list_text.SelectItemManual(i)
                        return
        except UnicodeDecodeError:
            print "UnicodeDecodeError!"
            print "Line num: ", i
            print "Line text: ", repr(line)
        self.frame.statusbar.SetStatusText("Text \"" + textToMatch + "\" not found.")

    # ---------
    # Start other misc functions
    # ---------
    def keyListener(self, event):
        keycode = event.GetKeyCode()
        if keycode == wx.WXK_DELETE or keycode == wx.WXK_BACK: # delete or backspace
            self.buttonUnmapSoundFromText(event)
        elif keycode == wx.WXK_INSERT:
            self.buttonMapSoundToText(event)

    def markAsDirty(self, boolToSet):
        if ssglob.lDirty[boolToSet] != True:
            self.dirtyCount += 1
            ssglob.lDirty[boolToSet] = True
            if self.frame.GetTitle()[-1] != '*':
                self.frame.SetTitle(self.frame.GetTitle() + '*')

    def markAsClean(self, boolToSet):
        if ssglob.lDirty[boolToSet] != False:
            self.dirtyCount -= 1
            ssglob.lDirty[boolToSet] = False
            if self.dirtyCount == 0 and self.frame.GetTitle()[-1] == '*':
                self.frame.SetTitle(self.frame.GetTitle()[:-1])

    def closeFrame(self, closeevent):
        if closeevent.CanVeto():
            if True in ssglob.lDirty:
                wd = wx.MessageDialog(self.frame,
                                      "Save modified files before quitting?",
                                      "ScummSpeaks",
                                      wx.YES_NO|wx.CANCEL|wx.ICON_EXCLAMATION)
                test = wd.ShowModal()
                if test == wx.ID_CANCEL:
                    return
                elif test == wx.ID_YES:
                    if ssglob.lDirty[ssglob.iTextDirty]:
                        self.menuSaveText(None)
                    if ssglob.lDirty[ssglob.iSpeechMapDirty]:
                        self.menuSaveSpeechMap(None)
                    # User could click "Cancel" during any "Save As..." dialog,
                    #  so again check if things are (still) dirty.
                    if True in ssglob.lDirty:
                        return

            # Everything other than "Cancel" falls through to this
            self.frame.Close(True)
        else:
            wx.Window.DestroyChildren(self.frame)
            wx.Window.Destroy(self.frame)

    def checkDirtyText(self):
        """ Checks if text has been marked as dirty. If dirty, prompts user to save text file via message dialog.
        Returns True on save/no save, False on cancel."""
        if ssglob.lDirty[ssglob.iTextDirty]:
            wd = wx.MessageDialog(self.frame,
                                  "Save modified text data?",
                                  "ScummSpeaks",
                                  wx.YES_NO|wx.CANCEL|wx.ICON_EXCLAMATION)
            test = wd.ShowModal()
            if test == wx.ID_YES:
                self.menuSaveText(None)
                return True
            elif test == wx.ID_NO:
                return True
            else:
                return False
        return True
