from __future__ import with_statement # requires Python 2.5
import wx
import ssglob
from ui.ids import *

class NewSpeechMapDialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        self.text_path = None
        self.sound_resource_path = None

        kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, *args, **kwds)
        self.bOK = wx.Button(self, ID_BUTTON_OK, "OK")
        self.bCancel = wx.Button(self, ID_BUTTON_CANCEL, "Cancel")

        self.lTextFile = wx.StaticText(self, -1, "Text File", style=wx.ALIGN_CENTRE)
        self.tcTextFile = wx.TextCtrl(self, -1, "")
        self.bTextBrowse = wx.Button(self, ID_BUTTON_TEXT_BROWSE, "Browse")

        self.lResourceFile = wx.StaticText(self, -1, "Sound resource File", style=wx.ALIGN_CENTRE)
        self.tcResourceFile = wx.TextCtrl(self, -1, "")
        self.bResourceBrowse = wx.Button(self, ID_BUTTON_RESOURCE_BROWSE, "Browse")

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_BUTTON, self.buttonOK, id=ID_BUTTON_OK)
        self.Bind(wx.EVT_BUTTON, self.buttonCancel, id=ID_BUTTON_CANCEL)
        wx.EVT_BUTTON(self, ID_BUTTON_TEXT_BROWSE, self.buttonTextBrowse)


    def __set_properties(self):
        self.SetTitle("New Speech Map")

    def __do_layout(self):
        szMain = wx.BoxSizer(wx.VERTICAL)
        szBottom = wx.BoxSizer(wx.HORIZONTAL)
        szTextInfoContainer = wx.BoxSizer(wx.VERTICAL)
        szTextInfo = wx.BoxSizer(wx.HORIZONTAL)
        #szResourceInfo = wx.BoxSizer(wx.HORIZONTAL)

        # Middle row
        szTextInfoContainer.Add(self.lTextFile, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        szTextInfo.Add(self.tcTextFile, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        szTextInfo.Add(self.bTextBrowse, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        szTextInfoContainer.Add(szTextInfo, 1, wx.EXPAND, 0)
        szMain.Add(szTextInfoContainer, 1, wx.EXPAND, 0)


        #szMain.Add(szResourceInfo, 1, wx.EXPAND, 0)

        szBottom.Add(self.bOK, 0, wx.ALIGN_RIGHT, 0)
        szBottom.Add(self.bCancel, 0, wx.ALIGN_RIGHT, 0)
        szMain.Add(szBottom, 1, wx.ALIGN_RIGHT, 0)

        self.SetSizer(szMain)
        szMain.Fit(self)
        self.Layout()

    def buttonOK(self, event):
        self.EndModal(wx.ID_OK)

    def buttonCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

    def buttonTextBrowse(self, event):
        sd = wx.FileDialog(self,
                           "Open text",
                           wildcard="TXT files (*.txt)|*.txt|All Files|*.*",
                           style=wx.OPEN|wx.FILE_MUST_EXIST|wx.CHANGE_DIR)
        if sd.ShowModal() == wx.ID_OK:
            filename = sd.GetPath()
            self.tcTextFile.SetValue(filename)

    def getTextPath(self):
        return self.tcTextFile.GetValue()

class MonsterTypeDialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        # begin wxGlade: MonsterTypeDialog.__init__
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, *args, **kwds)
        self.rbMonsterType = wx.RadioBox(self, -1, "MONSTER.SOU Type", choices=["OGG", "MP3", "FLAC", "VOC"], majorDimension=0, style=wx.RA_SPECIFY_COLS)
        self.bOK = wx.Button(self, ID_BUTTON_OK, "OK")
        self.bCancel = wx.Button(self, ID_BUTTON_CANCEL, "Cancel")

        self.__set_properties()
        self.__do_layout()

        self.Bind(wx.EVT_BUTTON, self.buttonOK, id=ID_BUTTON_OK)
        self.Bind(wx.EVT_BUTTON, self.buttonCancel, id=ID_BUTTON_CANCEL)
        # end wxGlade

    def __set_properties(self):
        # begin wxGlade: MonsterTypeDialog.__set_properties
        self.SetTitle("New MONSTER.SOU")
        self.rbMonsterType.SetSelection(0)
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: MonsterTypeDialog.__do_layout
        szMain = wx.BoxSizer(wx.VERTICAL)
        szBottom = wx.BoxSizer(wx.HORIZONTAL)
        szMain.Add(self.rbMonsterType, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        szBottom.Add(self.bOK, 0, wx.ALIGN_RIGHT, 0)
        szBottom.Add(self.bCancel, 0, wx.ALIGN_RIGHT, 0)
        szMain.Add(szBottom, 1, wx.ALIGN_RIGHT, 0)
        self.SetSizer(szMain)
        szMain.Fit(self)
        self.Layout()
        # end wxGlade

    def buttonOK(self, event): # wxGlade: MonsterTypeDialog.<event_handler>
        self.EndModal(wx.ID_OK)

    def buttonCancel(self, event): # wxGlade: MonsterTypeDialog.<event_handler>
        self.EndModal(wx.ID_CANCEL)

    def getMonsterType(self):
        return self.rbMonsterType.GetStringSelection()

# end of class MonsterTypeDialog

class ExportTextScriptDialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        # begin wxGlade: ExportTextScriptDialog.__init__
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, *args, **kwds)
        self.lblStart = wx.StaticText(self, -1, "Start line #")
        self.scStart = wx.SpinCtrl(self, -1, "", min=0, max=(len(ssglob.gTextData) - 1))
        self.label_2 = wx.StaticText(self, -1, "End line #")
        self.scEnd = wx.SpinCtrl(self, -1, "", min=0, max=(len(ssglob.gTextData) - 1))
        self.cbIgnore = wx.CheckBox(self, -1, "Ignore descriptions containing:")
        self.tcIgnoreText = wx.TextCtrl(self, -1, "var")
        self.cbMustContain = wx.CheckBox(self, -1, "Description must contain:")
        self.tcMustContainText = wx.TextCtrl(self, -1, "")
        self.bOK = wx.Button(self, wx.ID_OK, "OK")
        self.bCancel = wx.Button(self, wx.ID_CANCEL, "Cancel")

        self.lTextFile = wx.StaticText(self, -1, "Text File", style=wx.ALIGN_CENTRE)
        self.tcTextFile = wx.TextCtrl(self, -1, "")
        self.bTextBrowse = wx.Button(self, ID_BUTTON_TEXT_BROWSE, "Browse")

        wx.EVT_BUTTON(self, ID_BUTTON_TEXT_BROWSE, self.buttonTextBrowse)

        self.__set_properties()
        self.__do_layout()
        # end wxGlade

    def __set_properties(self):
        # begin wxGlade: ExportTextScriptDialog.__set_properties
        self.SetTitle("Export Text Script")
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: ExportTextScriptDialog.__do_layout
        szMain = wx.BoxSizer(wx.VERTICAL)
        szBottom = wx.BoxSizer(wx.HORIZONTAL)
        szMiddle = wx.BoxSizer(wx.HORIZONTAL)
        szMiddleRight = wx.BoxSizer(wx.VERTICAL)
        szMiddleLeft = wx.BoxSizer(wx.VERTICAL)
        szTop = wx.BoxSizer(wx.HORIZONTAL)
        szTopRight = wx.BoxSizer(wx.VERTICAL)
        szTopMid = wx.BoxSizer(wx.VERTICAL)
        szTopLeft = wx.BoxSizer(wx.VERTICAL)

        szTopLeft.Add(self.lblStart, 0, 0, 0)
        szTopLeft.Add(self.scStart, 0, 0, 0)
        szTop.Add(szTopLeft, 1, wx.EXPAND, 0)

        szTopMid.Add(self.label_2, 0, 0, 0)
        szTopMid.Add(self.scEnd, 0, 0, 0)
        szTop.Add(szTopMid, 1, wx.EXPAND, 0)

        szTopRight.Add(self.cbIgnore, 0, wx.ALIGN_RIGHT, 0)
        szTopRight.Add(self.tcIgnoreText, 0, wx.ALIGN_RIGHT, 0)
        szTop.Add(szTopRight, 1, wx.EXPAND, 0)

        szMain.Add(szTop, 1, wx.EXPAND, 0)

        # Middle row
        szMiddleLeft.Add(self.lTextFile, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        szMiddleLeft.Add(self.tcTextFile, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        szMiddleLeft.Add(self.bTextBrowse, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        szMiddle.Add(szMiddleLeft, 1, wx.EXPAND, 0)

        szMiddleRight.Add(self.cbMustContain, 0, wx.ALIGN_RIGHT, 0)
        szMiddleRight.Add(self.tcMustContainText, 0, wx.ALIGN_RIGHT, 0)
        szMiddle.Add(szMiddleRight, 1, wx.EXPAND, 0)

        szMain.Add(szMiddle, 1, wx.EXPAND, 0)

        # Bottom row
        szBottom.Add(self.bOK, 0, 0, 0)
        szBottom.Add(self.bCancel, 0, 0, 0)
        szMain.Add(szBottom, 1, wx.ALIGN_RIGHT, 0)
        self.SetSizer(szMain)
        szMain.Fit(self)
        self.Layout()
        # end wxGlade

    def buttonTextBrowse(self, event):
        sd = wx.FileDialog(self,
                           "Save output text",
                           wildcard="TXT files (*.txt)|*.txt|All Files|*.*",
                           style=wx.SAVE|wx.OVERWRITE_PROMPT|wx.CHANGE_DIR)
        if sd.ShowModal() == wx.ID_OK:
            filename = sd.GetPath()
            self.tcTextFile.SetValue(filename)

    def getRange(self):
        return (self.scStart.GetValue(), self.scEnd.GetValue())

    def ignoreText(self):
        """ Returns None if text should not be ignored, otherwise a string. """
        if self.cbIgnore.IsChecked():
            return self.tcIgnoreText.GetValue()
        else:
            return None

    def containsText(self):
        """ Returns None if descriptions don't have to contain a certain string, otherwise a string. """
        if self.cbMustContain.IsChecked():
            return self.tcMustContainText.GetValue()
        else:
            return None

    def getTextFilename(self):
        return self.tcTextFile.GetValue()

# end of class ExportTextScriptDialog

class ExportTextDialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, *args, **kwds)

        self.lbl_match = wx.StaticText(self, -1, "Export lines with description containing:", style=wx.ALIGN_CENTRE)
        self.tc_match = wx.TextCtrl(self, -1, "")

        self.label_text_file = wx.StaticText(self, -1, "Text File", style=wx.ALIGN_CENTRE)
        self.text_ctrl_text_file = wx.TextCtrl(self, -1, "")
        self.button_text_browse = wx.Button(self, ID_BUTTON_TEXT_BROWSE, "Browse") # can we use the same ID twice?

        self.panel_1 = wx.Panel(self, -1)
        self.button_ok = wx.Button(self, ID_BUTTON_OK, "OK")
        self.button_cancel = wx.Button(self, ID_BUTTON_CANCEL, "Cancel")

        self.__set_properties()
        self.__do_layout()

        wx.EVT_BUTTON(self, ID_BUTTON_OK, self.buttonOK)
        wx.EVT_BUTTON(self, ID_BUTTON_CANCEL, self.buttonCancel)
        wx.EVT_BUTTON(self, ID_BUTTON_TEXT_BROWSE, self.buttonTextBrowse)

    def __set_properties(self):
        self.SetTitle("Export Text")
        self.lbl_match.SetSize((200, 15))
        self.tc_match.SetSize((150, 21))
        self.label_text_file.SetSize((60, 15))
        self.text_ctrl_text_file.SetSize((150, 21))

    def __do_layout(self):
        main_sizer = wx.BoxSizer(wx.VERTICAL)
        sizer_text_file = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1 = wx.BoxSizer(wx.HORIZONTAL)

        main_sizer.Add(self.lbl_match, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        main_sizer.Add((5, 5), 0, wx.FIXED_MINSIZE, 0)
        main_sizer.Add(self.tc_match, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        main_sizer.Add((20, 20), 0, wx.FIXED_MINSIZE, 0)

        sizer_text_file.Add(self.label_text_file, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        sizer_text_file.Add(self.text_ctrl_text_file, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        sizer_text_file.Add(self.button_text_browse, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        main_sizer.Add(sizer_text_file, 1, wx.EXPAND, 0)

        sizer_1.Add(self.panel_1, 1, wx.EXPAND, 0)
        sizer_1.Add(self.button_ok, 0, wx.ALIGN_BOTTOM|wx.FIXED_MINSIZE, 0)
        sizer_1.Add(self.button_cancel, 0, wx.ALIGN_BOTTOM|wx.FIXED_MINSIZE, 0)
        main_sizer.Add(sizer_1, 1, wx.EXPAND, 0)

        self.SetAutoLayout(1)
        self.SetSizer(main_sizer)
        main_sizer.Fit(self)
        main_sizer.SetSizeHints(self)
        self.Layout()

    def buttonOK(self, event):
        """ Does nothing other than close the dialog. Actual saving is performed
        in the code that initially displayed this dialog."""
        self.EndModal(wx.ID_OK)

    def buttonCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

    def buttonTextBrowse(self, event):
        sd = wx.FileDialog(self,
                           "Save output text",
                           wildcard="TXT files (*.txt)|*.txt|All Files|*.*",
                           style=wx.SAVE|wx.OVERWRITE_PROMPT|wx.CHANGE_DIR)
        if sd.ShowModal() == wx.ID_OK:
            filename = sd.GetPath()
            self.text_ctrl_text_file.SetValue(filename)

    def getMatchText(self):
        return self.tc_match.GetValue()

    def getTextFilename(self):
        return self.text_ctrl_text_file.GetValue()


class FindTextDialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        # This is ripped from DRO Trimmer
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, *args, **kwds)

        self.parent = args[0]

        self.lTTF = wx.StaticText(self, -1, "Text to find:")
        self.tcSearchText = wx.TextCtrl(self, -1, "")
        self.cbSearchDesc = wx.CheckBox(self, -1, "Search descriptions")
        self.bOk = wx.Button(self, ID_BUTTON_FINDNEXT, "Find Next")
        self.bCancel = wx.Button(self, wx.ID_CANCEL, "Close")

        wx.EVT_BUTTON(self, ID_BUTTON_FINDNEXT, self.parent.parent.buttonFindText)

        self.__set_properties()
        self.__do_layout()

    def __set_properties(self):
        self.SetTitle("Find Text")

    def __do_layout(self):
        sMain = wx.BoxSizer(wx.VERTICAL)
        sBottom = wx.BoxSizer(wx.HORIZONTAL)

        gsTop = wx.FlexGridSizer(1, 2, 0, 0)
        gsTop.Add(self.lTTF, 0, 0, 0)
        gsTop.Add(self.tcSearchText, 0, 0, 0)
        gsTop.Add(self.cbSearchDesc, 0, wx.ALIGN_RIGHT, 0)
        sMain.Add(gsTop, 1, wx.EXPAND, 0)

        sBottom.Add(self.bOk, 0, 0, 0)
        sBottom.Add(self.bCancel, 0, wx.LEFT, 10)
        sMain.Add(sBottom, 0, wx.ALL|wx.ALIGN_RIGHT, 5)

        self.SetSizer(sMain)
        sMain.Fit(self)
        self.Layout()

    def getSearchText(self):
        return self.tcSearchText.GetValue()

    def shouldSearchDesc(self):
        return self.cbSearchDesc.IsChecked()
