import wx
from ui.ids import *

class ExportTextDialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE
        wx.Dialog.__init__(self, *args, **kwds)

        self.lbl_match = wx.StaticText(self, -1, "Export lines with description containing:", style=wx.ALIGN_CENTRE)
        self.tc_match = wx.TextCtrl(self, -1, "")

        self.label_text_file = wx.StaticText(self, -1, "Text File", style=wx.ALIGN_CENTRE)
        self.text_ctrl_text_file = wx.TextCtrl(self, -1, "")
        self.button_text_browse = wx.Button(self, ID_BUTTON_TEXT_BROWSE, "Browse") # can we use the same ID twice?

        self.panel_1 = wx.Panel(self, -1)
        self.button_ok = wx.Button(self, ID_BUTTON_OK, "OK")
        self.button_cancel = wx.Button(self, ID_BUTTON_CANCEL, "Cancel")

        self.__set_properties()
        self.__do_layout()

        wx.EVT_BUTTON(self, ID_BUTTON_OK, self.buttonOK)
        wx.EVT_BUTTON(self, ID_BUTTON_CANCEL, self.buttonCancel)
        wx.EVT_BUTTON(self, ID_BUTTON_TEXT_BROWSE, self.buttonTextBrowse)

    def __set_properties(self):
        self.SetTitle("Export Text")
        self.lbl_match.SetSize((200, 15))
        self.tc_match.SetSize((150, 21))
        self.label_text_file.SetSize((60, 15))
        self.text_ctrl_text_file.SetSize((150, 21))

    def __do_layout(self):
        main_sizer = wx.BoxSizer(wx.VERTICAL)
        sizer_text_file = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1 = wx.BoxSizer(wx.HORIZONTAL)

        main_sizer.Add(self.lbl_match, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        main_sizer.Add((5, 5), 0, wx.FIXED_MINSIZE, 0)
        main_sizer.Add(self.tc_match, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        main_sizer.Add((20, 20), 0, wx.FIXED_MINSIZE, 0)

        sizer_text_file.Add(self.label_text_file, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        sizer_text_file.Add(self.text_ctrl_text_file, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        sizer_text_file.Add(self.button_text_browse, 0, wx.ALIGN_CENTER_VERTICAL|wx.FIXED_MINSIZE, 0)
        main_sizer.Add(sizer_text_file, 1, wx.EXPAND, 0)

        sizer_1.Add(self.panel_1, 1, wx.EXPAND, 0)
        sizer_1.Add(self.button_ok, 0, wx.ALIGN_BOTTOM|wx.FIXED_MINSIZE, 0)
        sizer_1.Add(self.button_cancel, 0, wx.ALIGN_BOTTOM|wx.FIXED_MINSIZE, 0)
        main_sizer.Add(sizer_1, 1, wx.EXPAND, 0)

        self.SetAutoLayout(1)
        self.SetSizer(main_sizer)
        main_sizer.Fit(self)
        main_sizer.SetSizeHints(self)
        self.Layout()

    def buttonOK(self, event):
        """ Does nothing other than close the dialog. Actual saving is performed
        in the code that initially displayed this dialog."""
        self.EndModal(wx.ID_OK)

    def buttonCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

    def buttonTextBrowse(self, event):
        sd = wx.FileDialog(self,
                           "Save output text",
                           wildcard="TXT files (*.txt)|*.txt|All Files|*.*",
                           style=wx.SAVE|wx.OVERWRITE_PROMPT|wx.CHANGE_DIR)
        if sd.ShowModal() == wx.ID_OK:
            filename = sd.GetPath()
            self.text_ctrl_text_file.SetValue(filename)

    def getMatchText(self):
        return self.tc_match.GetValue()

    def getTextFilename(self):
        return self.text_ctrl_text_file.GetValue()
